@@ -1,4 +1,4 @@
-package WebService::FreeDB;
+package WebService::FreeDB; # -*- tab-width:8 -*- 
 use Data::Dumper;
 use LWP::UserAgent; # Erweiterung jb
 
@@ -6,7 +6,7 @@ require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw//;
 @EXPORT_OK = qw/getdiscs getdiscinfo ask4discurls outdumper outstd/;
-$VERSION = '0.77';
+$VERSION = '0.78';
 
 #####
 # Description: for getting a instace of this Class
@@ -93,72 +93,24 @@ sub getdiscs {
 	if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 2) {
 		print STDERR "**url-search: $url;\n" ;
 	}
-	
-	my $ua = LWP::UserAgent->new();				
+
+	my $ua = LWP::UserAgent->new();
 	$ua->proxy('http' => $self->{ARG}->{PROXY});
-	my $req = HTTP::Request->new(GET => $url);	
-	my $response = $ua->request($req);			
-	if ($response->is_success) {				
-		my $data = $response->content;			
+	my $req = HTTP::Request->new(GET => $url);
+	my $response = $ua->request($req);
+	if ($response->is_success) {
+		my $data = $response->content;
 		@lines = split /\n/, $data ;
 		my $liststart = 0 ;
 		my $lastref;
-		for my $line (@lines) {
-		    #if($line =~ /^<h2>all categories<\/h2>$/) {  #update at 31.3.06: changed this
-			if($line =~ /^<h2>freedb music CD search results<\/h2>$/) {
-				$liststart=1;
-			} elsif ( $liststart == 1 && $line =~ /<table border=0/){
-				if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 2) {
-					print STDERR "**list start found;\n";
-				}
-				$liststart=2;
-			} elsif ( $liststart == 2 && $line =~ /<tr><td><a href="(.+)">(.+) \/ (.+)<\/a> &nbsp; <a href="(.+)"><font size=-1>\d+<\/font><\/a>/ ) {
- 			  	if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 3) {
-					print STDERR "***multilist element found $url;\n";
-				}
-				if(!(defined($discs{$1}))) {
-					$discs{$1} = [$2,$3,$4];
-					$lastref = $1;
-				} else {
-					if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-				   		print STDERR "*already got an disc, taking old one !: $line;\n";
-					}
-				}
-				#} elsif ( $liststart == 2 && $line =~ /^<tr><td><a href="(.+)">(.+) \/ (.+)<\/a><br>/ ) {
-			} elsif ( $liststart == 2 && $line =~ /<tr><td><b>.*<a href="(.+)">(.+) \/ (.+)<\/a>/ ) {
-  				if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 3) {
-					print STDERR "***list element found $url;\n";
-				}
-				if(!(defined($discs{$1}))) {
-					$discs{$1} = [$2,$3];
-					$lastref = undef;
-				} else {
-  					if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-						print STDERR "*already got an disc, taking old one !: $line;\n";
-					}
-				}
-			} elsif (defined($lastref) && $liststart == 2 && $line =~ /^<a href="(.+)"><font size=-1>\d+<\/font><\/a>/ ) {
-				if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 3) {
-					print STDERR "***more multilist element found $url;\n" ;
-				}
-				if(!(defined($discs{$lastref}))) {
-					if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-						print STDERR "*but no lastref-element found $url;\n";
-					}
-				} else {
-					push(@{$discs{$lastref}},$1);
-				}
-			} elsif ( $liststart == 2 && $line =~ /^<\/table>$/) {
-				if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 2) {
-					print STDERR "**list end found;\n";
-				}
-				$liststart = 0;
-			} elsif ($liststart == 2 ) {
-				if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 3) {
-					print STDERR "***unknown line-type, ignoring : $line;\n";
-				}
-			}
+		my $line = shift @lines;
+		while (!($line =~ m|^<a name="fdbsr"></a>|)) {
+		  $line = shift @lines; 
+		  last unless $line;
 		}
+		die "no match" unless $line;
+                $discs{$1} = [$2,$3]
+		  while $line =~ m|<a href="(.+?)" class=searchResultTopLinkA .+? title=".+? / .+?">(.+?) / (.+?)</a></td>|g;
 	}							   
 	else {						   
 		die $response->status_line;
@@ -176,16 +128,16 @@ sub getdiscinfo {
 	my $self = shift;
 	my $url = shift;
 	my %disc;
-	
+
 	if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 2) {
 		print STDERR "**url-disc:$url;\n";
 	}
-	my $ua = LWP::UserAgent->new();				
+	my $ua = LWP::UserAgent->new();
 	$ua->proxy('http' => $self->{ARG}->{PROXY});
-	my $req = HTTP::Request->new(GET => $url);	
-	my $response = $ua->request($req);			
-	if ($response->is_success) {				
-		my $data = $response->content;			
+	my $req = HTTP::Request->new(GET => $url);
+	my $response = $ua->request($req);
+	if ($response->is_success) {
+		my $data = $response->content;
 		if (!defined($data)) {
 			if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
 				print STDERR "*found no disc;\n";
@@ -195,64 +147,63 @@ sub getdiscinfo {
 		$disc{url} = $url;
 		@lines = split(/\n/,$data);
 		$line = shift(@lines);
-		#ignore until begin of data
-		while (!($line =~ /^<table width="100%" border="0" cellspacing="1" cellpadding="8" bgcolor="#FFFFFF"><tr><td>$/)) {
-			$line = shift(@lines);
+		#ignore until begin of searchResult data
+		while (!($line =~ m|^<a name="fdbsr"></a>|)) {
+		  $line = shift(@lines);
+		  last unless $line;
 		}
 		if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 2) {
 			print STDERR "**found start of data :$line;\n"; 
 		}
-		if ($lines[0] =~ /^<h2>(.+) \/ (.+)<\/h2>$/) {
+		die "found no data. FreeDB template format changed?\n" unless $line;
+		if ($line =~ m|/id="searchU11" title="(.+?) / (.+?)">|) {
 			$disc{artist} = $1;
 			$disc{album} = $2;
 		} else {
 			if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-				print STDERR "*format error(artist+album):$lines[0];\n";
+				print STDERR "*format error(artist+album):$line;\n";
 			}
 		}
-		#ignore commercials
-		while (!($lines[1] =~ /^\s*tracks:\s*?(\d+)<br>$/)) {
-			shift(@lines);
-		}
-
-		if ($lines[1] =~ /^\s*tracks:\s*?(\d+)<br>$/) {
+		if ($line =~ m|<b>Tracks:</b>\s*?(\d+)<br>|) {
 			$disc{tracks} = $1;
 		} else {
   			if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-				print STDERR "*format error(tracks):$lines[1];\n";
+				print STDERR "*format error(tracks):$line;\n";
 			}
 		}
-		if ($lines[2] =~ /^total time:\s*(\d+:\d+)<br>$/) {
+		if ($line =~ m|<b>Total time:</b>\s*(\d+:\d+)<br>|) {
 			$disc{totaltime} = $1;
 		} else {
 			if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-	 			print STDERR "*format error(totaltime):$lines[2];\n";
+	 			print STDERR "*format error(totaltime):$line;\n";
 			}
 		}
-		if ($lines[3] =~ /^year:\s*(\d*)<br>$/) {
+		if ($line =~ m|<b>Year:</b>\s*(\d*)<br>|) {
 			$disc{year} = $1;
 		} else {
 			 if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-				print STDERR "*format error(year):$lines[3];\n";
+				print STDERR "*format error(year):$line;\n";
 			}
 		}
-		if ($lines[4] =~ /^genre:\s*(.*)<br>$/) {
+		if ($line =~ m|<b>Disc-ID:</b>\s*(.*?) / |) {
 			$disc{genre} = $1;
 		} else {
   			if (defined $self->{ARG}->{DEBUG} && $self->{ARG}->{DEBUG} >= 1) {
-				print STDERR "*format error(genre):$lines[4];\n";
+				print STDERR "*format error(genre):$line;\n";
 			}
 		}
 		if(!defined($disc{artist})) {$disc{artist} = "";}
 		if(!defined($disc{album})) {$disc{album} = "";}
 		if(!defined($disc{year})) {$disc{year} = "";}
 		if(!defined($disc{genre})) {$disc{genre} = "";}
+
 		while (!($line =~ /^<table border=0>$/)) { #ignore until begin of tackinfo
 			if ($line =~ /^<br><hr><center><table width="98%"><tr><td bgcolor="#E8E8E8"><pre>$/) {
 				$line = shift(@lines);
 				while (!($line =~ /<\/pre><\/tr><\/td><\/table><\/center>/)) {
 					$disc{rest} .= $line."\n";
 					$line = shift(@lines);
+					last unless $line;
 				}
 			}
 			$line = shift(@lines);
@@ -283,13 +234,13 @@ sub getdiscinfo {
 					print STDERR "**found end of trackinfo & data: $line;\n";
 				}
 			} else {next;}
-		
+
 		}
 		return %disc;
-	} else {							
+	} else {
 	  #print STDERR "Url was: ".$url."\n";
-		die $response->status_line;	
-	}								
+		die $response->status_line;
+	}
 }
 
 #####
@@ -494,13 +445,13 @@ sub outxml {
 #####
 sub ascii2xml {
 	$ascii = $_[0];
-	
+
 	$ascii =~ s/&/&amp;/g;
 	$ascii =~ s/</&lt;/g;
 	$ascii =~ s/>/&gt;/g;
 	$ascii =~ s/'/&apos;/g;
 	$ascii =~ s/"/&quot;/g;
-	
+
 	return $ascii;
 }
 
